/* Copyright 1999 Jon Dart. All Rights Reserved. */

#include "hash.h"
#include "util.h"
#include "debug.h"
//#include <malloc.h>
#include <memory.h>
#include <stddef.h>

static Dual_Position_Info *hashTable = NULL;
static int hashSize;
static const int MaxRehash = 2;
static int refCount = 0;

void initHash(int size)
{
   hashSize = size/2 + (size/2 % 2); // assure it's even
   hashTable = (Dual_Position_Info*)malloc(sizeof(Dual_Position_Info)*hashSize);
   memset(hashTable,'\0',sizeof(Dual_Position_Info)*hashSize);
   ++refCount;
}

void freeHash()
{
   if (--refCount == 0)
   {
     free(hashTable);
   }
}

Position_Info *searchHash(const Board &currentBoard, int rep_count)
{
   Position_Info look(currentBoard,rep_count);
   hash_t probe = quickmod(look.hash_code(),hashSize);
   Dual_Position_Info *p = (Dual_Position_Info*)&(hashTable[probe]);
   if (p->replace.is_equal(look)) 
   {
      return &p->replace;
   }
   for (int i = MaxRehash+1; i>0; --i)
   {
      if (p->depth.is_equal(look))
      {
         return &p->depth;
      }
      probe++; p++;
      if (probe >= hashSize) {
        probe = 0; p = hashTable;      
      }
   }
   return NULL;
}

int storeHash(hash_t hash_code, Position_Info &newPI)
{
   int probe = quickmod(hash_code,hashSize);
   Dual_Position_Info *p = (Dual_Position_Info*)&(hashTable[probe]);
   p->replace = newPI;

   Position_Info *best = NULL;
   int minDepth = 10000;
   // Of the positions that hash to the same locations
   // as this one, find the best one to replace.
   for (int i = 0; i < MaxRehash+1; i++)
   {
      Position_Info *q = &p->depth;
      
      if (q->hash_code() == (hash_t)0 || q->age() != newPI.age())
      {
         *q = newPI;
         return 0;
      }
      if (q->hash_code() == hash_code) {
         // same position
		 q->update_best_move(newPI);
      }
      if (q->depth() < minDepth)
      {
         minDepth = q->depth();
         if (newPI.depth() > minDepth) best = q;
      }
      probe++; p++;
      if (probe >= hashSize) {
        probe = 0; p = hashTable;      
      }
   }
   if (best != NULL) {
      *best = newPI;
      return 1; /* replaced */
   }
   else
   {
      return 2; /* fail*/
   }
}

