// Copyright 2001 Jon Dart. All Rights Reserved.
//
// This module interfaces between the Nalimov tb code
// and Arasan's databatypes

#if defined(_MSC_VER)
#pragma optimize("g",off)
#endif

#define INDEX unsigned long
typedef int square;
typedef int piece;
#define NEW
#define XX 64
#define C_PIECES 3
#define T41_INCLUDE

#define SqFindKing(psq)      (psq[C_PIECES*(King-1)])
#define SqFindOne(psq, p)    (psq[C_PIECES*(p-1)])
#define SqFindFirst(psq, p)  (psq[C_PIECES*(p-1)])
#define SqFindSecond(psq, p) (psq[C_PIECES*(p-1)+1])
#define SqFindThird(psq, p)  (psq[C_PIECES*(p-1)+2])

#include "tbprobe.h"
#include "constant.h"
//#include <malloc.h>
#include <stdio.h>

#include "tbindex.cpp"

// Arasan's board is layed out differently from
// most chess programs (and what the tbs assume).
// This table is used to correct for this:

static const Square flip[64] =
{
        A1, B1, C1, D1, E1, F1, G1, H1,
        A2, B2, C2, D2, E2, F2, G2, H2,
        A3, B3, C3, D3, E3, F3, G3, H3,
        A4, B4, C4, D4, E4, F4, G4, H4,
        A5, B5, C5, D5, E5, F5, G5, H5,
        A6, B6, C6, D6, E6, F6, G6, H6,
        A7, B7, C7, D7, E7, F7, G7, H7,
        A8, B8, C8, D8, E8, F8, G8, H8
};

int initTB(char *tbpath,int tb_cache_size) {
    TB_CRC_CHECK = 0;
    int EGTBMenCount = IInitializeTb(tbpath);
    if(EGTBMenCount != 0) {
        void *egtb_cache = malloc(tb_cache_size*1024);
        if (!egtb_cache) 
	  cerr << "Warning: failed to allocate memory for TB cache!" << endl; 
        //cout << "Found " << EGTBMenCount << "-men endgame table bases." << endl;
        FTbSetCacheSize(egtb_cache, tb_cache_size*1024);
    }
    return EGTBMenCount;
}

static inline void initSqCtrs(int *counter,int *squares,PieceType p,const Bitmap &bits) {
  int  cPieces=0;
  Bitmap b(bits);
  while (!b.is_clear()) {
    const Square sq=b.first_one();
    ASSERT((p-1)*C_PIECES+cPieces<16);
    squares[(p-1)*C_PIECES+cPieces]=flip[sq];
    cPieces++;
    b.clear(sq);
  }
  *counter=cPieces;
}

int probe_tb(const Board &board, int &score, int ply) {

    int pcCount[10];
    int wSquares[16], bSquares[16];
    int iTB;
    int color;
    int invert;
    int *wp, *bp;
    int ep;
    INDEX index;
    int value;

    initSqCtrs(pcCount,   wSquares, Pawn, board.pawn_bits[White]);
    initSqCtrs(pcCount+1, wSquares, Knight, board.knight_bits[White]);
    initSqCtrs(pcCount+2, wSquares, Bishop, board.bishop_bits[White]);
    initSqCtrs(pcCount+3, wSquares, Rook, board.rook_bits[White]);
    initSqCtrs(pcCount+4, wSquares, Queen, board.queen_bits[White]);
    initSqCtrs(pcCount+5, bSquares, Pawn, board.pawn_bits[Black]);
    initSqCtrs(pcCount+6, bSquares, Knight, board.knight_bits[Black]);
    initSqCtrs(pcCount+7, bSquares, Bishop, board.bishop_bits[Black]);
    initSqCtrs(pcCount+8, bSquares, Rook, board.rook_bits[Black]);
    initSqCtrs(pcCount+9, bSquares, Queen, board.queen_bits[Black]);
    wSquares[15] = flip[board.KingPos(White)];
    bSquares[15] = flip[board.KingPos(Black)];

    iTB = IDescFindFromCounters(pcCount);
    if(iTB == 0) {
	return 0;
    }

    if(iTB > 0) {
        color = board.Side() == White ? 0 : 1;
	invert = 0;
	wp = wSquares;
	bp = bSquares;
    } else {
 	color = board.Side() == White ? 1 : 0;
	invert = 1;
	wp = bSquares;
	bp = wSquares;
	iTB = - iTB;
    }

    if(!FRegisteredFun(iTB, color)) {
      return 0;
    }

    Square epsq = board.EnPassantSq();
    ep  = (epsq == InvalidSquare) ? XX : flip[Square(File(epsq),3,board.Side())];
    index = PfnIndCalcFun(iTB, color) (wp, bp, ep, invert);
    value = L_TbtProbeTable(iTB, color, index);
    if(value == bev_broken) {
      // illegal position
	return 0;
    }

    //#ifdef _TRACE
    //    indent(ply); cout << " TB hit: " << board << " score= " << value <<endl;
    //#endif
    if(value > 0) {
	value = ((value-L_bev_mi1-1)*2+Constants::BIG-ply);
    }
    else if(value < 0) {
	value = ((value+L_bev_mi1)*2 - Constants::BIG+ply);
    }
    score = value;
    return 1;

}
